@@ -1,3 +1,4 @@
+{-# OPTIONS -fglasgow-exts #-}
 {-
     BNF Converter: Abstract syntax
     Copyright (C) 2004  Author:  Markus Forberg, Michael Pellauer, Aarne Ranta
@@ -22,6 +23,7 @@ module CF (
 	    CF,
 	    Rule,
 	    Pragma(..),
+	    Exp(..),
 	    Literal,
 	    Symbol,
 	    KeyWord,
@@ -39,6 +41,8 @@ module CF (
 	    specialCatsP,   -- all literals
 	    specialData,    -- special data
 	    isCoercion,     -- wildcards in grammar (avoid syntactic clutter)
+	    isDefinedRule,  -- defined rules (allows syntactic sugar)
+	    isProperLabel,  -- not coercion or defined rule
 	    allCats,        -- all categories of a grammar
 	    allCatsIdNorm,
 	    allEntryPoints, -- those categories that are entry points to the parser
@@ -68,6 +72,7 @@ module CF (
 	    rhsRule,        -- The list of Terminals/NonTerminals of a rule.
 	    normCat,        -- Removes precendence information. C1 => C, [C2] => [C]
 	    normCatOfList,  --   Removes precendence information and enclosed List. C1 => C, C2 => C
+	    catOfList,	    -- Removes enclosed list: [C1] => C1
 	    comments,       -- translates the pragmas into two list containing the s./m. comments
             tokenPragmas,   -- user-defined regular expression tokens
             tokenNames,     -- The names of all user-defined tokens
@@ -121,6 +126,41 @@ type Exts = ([Pragma],Info)
 -- KeyWord  - reserved words, e.g. 'if' 'while'
 type Info = ([Literal],[Symbol],[KeyWord],[Cat])
 
+-- Expressions for function definitions
+data Exp = App String [Exp]
+	 | LitInt Integer
+	 | LitDouble Double
+	 | LitChar Char
+	 | LitString String
+
+instance Show Exp where
+    showsPrec p e =
+	case listView e of
+	    Right es	->
+		showString "["
+		. foldr (.) id (intersperse (showString ", ") $ map shows es)
+		. showString "]"
+	    Left (App x []) -> showString x
+	    Left (App "(:)" [e1,e2]) ->
+		showParen (p>0)
+		$ showsPrec 1 e1
+		. showString " : "
+		. shows e2
+	    Left (App x es) ->
+		showParen (p>1)
+		$ foldr (.) id
+		$ intersperse (showString " ")
+		$ showString x : map (showsPrec 2) es
+	    Left (LitInt n)	-> shows n
+	    Left (LitDouble x)	-> shows x
+	    Left (LitChar c)	-> shows c
+	    Left (LitString s)	-> shows s
+	where
+	    listView (App "[]" []) = Right []
+	    listView (App "(:)" [e1,e2])
+		| Right es <- listView e2   = Right $ e1:es
+	    listView e	= Left e
+
 -- pragmas for single line comments and for multiple-line comments.
 data Pragma = CommentS  String        
             | CommentM (String,String)
@@ -129,7 +169,9 @@ data Pragma = CommentS  String
             | Layout [String]
             | LayoutStop [String]
             | LayoutTop
+	    | FunDef String [String] Exp
             -- ...
+	      deriving (Show)
 
 tokenPragmas :: CFG f -> [(String,Reg)]
 tokenPragmas cf = [(name,exp) | TokenReg name _ exp <- pragmasOfCF cf]
@@ -298,6 +340,7 @@ prTree (Tree (fun,trees)) = fun +++ unwords (map pr2 trees) where
 cf2data :: CF -> [Data]
 cf2data cf = 
   [(cat, nub (map mkData [r | r@(f,_) <- rulesOfCF cf, 
+			      not (isDefinedRule f),
                               not (isCoercion f), eqCat cat (valCat r)])) 
       | cat <- allNormalCats cf] 
  where
@@ -308,6 +351,7 @@ cf2data cf =
 cf2dataLists :: CF -> [Data]
 cf2dataLists cf = 
   [(cat, nub (map mkData [r | r@(f,_) <- rulesOfCF cf, 
+			      not (isDefinedRule f),
                               not (isCoercion f), eqCat cat (valCat r)])) 
       | cat <- (filter (\x -> not $ isDigit $ last x) (allCats cf))] 
  where
@@ -328,6 +372,12 @@ allNormalCats = filter isNormal . allCats
 isCoercion :: Fun -> Bool
 isCoercion = (== "_")
 
+isDefinedRule :: Fun -> Bool
+isDefinedRule (x:_) = isLower x
+
+isProperLabel :: Fun -> Bool
+isProperLabel f = not (isCoercion f || isDefinedRule f)
+
 -- categories C1, C2,... (one digit in end) are variants of C
 
 eqCat :: Cat -> Cat -> Bool
@@ -468,8 +518,8 @@ checkRule cf r@((f,_),(cat,rhs))
    badTypeName = not (null badtypes)
    badtypes = filter isBadType $ cat : [c | Left c <- rhs]
    isBadType c = not (isUpper (head c) || isList c || c == "#")
-   badFunName = not (isUpper (head f) 
-                       || isCoercion f ||isNilFun f || isOneFun f || isConsFun f)
+   badFunName = not (all (\c -> isAlphaNum c || c == '_') f {-isUpper (head f)-}
+                       || isCoercion f || isNilFun f || isOneFun f || isConsFun f)
 
 isPositionCat :: CFG f -> Cat -> Bool
 isPositionCat cf cat =  or [b | TokenReg name b _ <- pragmasOfCF cf, name == cat]
