@@ -307,7 +307,7 @@ prPrintFun _ = ""
 --Generates methods for the Pretty Printer
 prPrintData :: [UserDef] -> (Cat, [Rule]) -> String
 prPrintData user (cat, rules) = 
- if "List" `isPrefixOf` (identCat cat)
+ if isList cat
  then unlines
  [
   "void pp" ++ cl ++ "("++ cl +++ vname ++ ", int i)",
@@ -345,9 +345,10 @@ prPrintData user (cat, rules) =
  ]
  where
    cl = identCat (normCat cat)
+   ecl = identCat (normCatOfList cat)
    vname = map toLower cl
-   member = (drop 4 vname) ++ "_"
-   visitMember = "      pp" ++ (drop 4 cl) ++ "(" ++ vname ++ "->" ++ member ++ ", 0);"
+   member = map toLower ecl
+   visitMember = "      pp" ++ ecl ++ "(" ++ vname ++ "->" ++ member ++ "_, 0);"
    (sc, sep) = if length sep' == 1
      then ("C", "'" ++ escapeChars sep' ++ "'")
      else ("S", "\"" ++ escapeChars sep' ++ "\"")
@@ -414,7 +415,7 @@ prShowFun _ = ""
 --This prints the functions for Abstract Syntax tree printing.
 prShowData :: [UserDef] -> (Cat, [Rule]) -> String
 prShowData user (cat, rules) = 
- if "List" `isPrefixOf` (identCat cat)
+ if isList cat
  then unlines
  [
   "void sh" ++ cl ++ "("++ cl +++ vname ++ ")",
@@ -451,9 +452,10 @@ prShowData user (cat, rules) =
  ]
  where
    cl = identCat (normCat cat)
+   ecl = identCat (normCatOfList cat)
    vname = map toLower cl
-   member = (drop 4 vname) ++ "_"
-   visitMember = "      sh" ++ (drop 4 cl) ++ "(" ++ vname ++ "->" ++ member ++ ");"
+   member = map toLower ecl
+   visitMember = "      sh" ++ ecl ++ "(" ++ vname ++ "->" ++ member ++ "_);"
     
 --Pretty Printer methods for a rule.
 prShowRule :: [UserDef] -> Rule -> String
